/*****************************************************/
/* A simple realization of growable array of strings */
/*****************************************************/

#include <windows.h>
#include "hem.h"

#define ALLOC_SIZE	4		/* number of free entries to add when the array grows */

/* Since we use static data, only one array per process is available */
static int Count = 0;
static int Available = 0;
static HEM_BYTE **lpArray = 0;

//
// Initialize array. Must be called first
//
void StrArrayInitialize()
{
	Count = 0;
	Available = 0;
	lpArray = 0;
}

//
// Free array
//
void StrArrayFree()
{
	if(lpArray)
	{
		/* free all strings */
		for(int i = 0; i < Count; i++) free(lpArray[i]);
		
		/* free the array of pointers itself */
		free(lpArray);
	}
	StrArrayInitialize();
}

//
// Grow the array when there is no space to add a string. This is an internal function not visible outside this module
//
static void StrArrayGrow()
{
	if(Available == 0)
	{
		int NewSize = (Count + ALLOC_SIZE) * sizeof (HEM_BYTE *);
		if(lpArray)
		{
			void *Tmp = realloc(lpArray, NewSize);
			if(!Tmp) StrArrayFree();
			lpArray = (HEM_BYTE **)Tmp;
		}
		else lpArray = (HEM_BYTE **)malloc(NewSize);
		if(!lpArray) RaiseException(STATUS_NO_MEMORY, EXCEPTION_NONCONTINUABLE, 0, 0);
		Available += ALLOC_SIZE;
	}
}

//
// Get number of strings
//
int StrArrayGetCount()
{
	return Count;
}

//
// Get array of strings
//
HEM_BYTE ** StrArrayGetArray()
{
	return lpArray;
}

//
// Add a string to the array
//
void StrArrayAdd(char *Str)
{
	StrArrayGrow();
	int Len = lstrlen(Str) + 1;	/* include terminating zero */
	lpArray[Count] = (HEM_BYTE *)malloc(Len);
	RtlMoveMemory(lpArray[Count], Str, Len);
	Count++;
	Available--;
}

//
// Get size of the largest string
//
int StrArrayGetWidth()
{
	int MaxLen = 0;
	for(int i = 0; i < Count; i++)
	{
		int CurLen = strlen((char *)lpArray[i]);
		if(CurLen > MaxLen) MaxLen = CurLen;
	}
	return MaxLen;
}